# NoHarvester.pl
# No Harvester v1.2.1 plugin for Movable Type
# http://www.chipple.net/mt/noharvester/
# by Patrick Benny http://www.chipple.net/
# This code cannot be redistributed without permission from the author. 
#
# Version history
#   1.2.1 - 2007/10/23 - Simplified addition to templates
#   1.2   - 2007/8/7   - Now supports Movable Type 4
#   1.1   - 2007/3/5   - Added support for forms generated with <MTCommentFields>
#   1.0.3 - 2007/2/22  - Fixed "Subroutine MT::App::Comments::post redefined" warning
#   1.0.2 - 2006/10/24 - Fixed Digest::MD5::md5_hex warning showing up in error log
#   1.0.1 - 2006/10/2  - Fixed PHP to not use "short tags" (no need to upgrade)
#   1.0   - 2006/9/26  - First public release
#   0.9   - 2006/6/16  - Personal version


use strict;
package MT::Plugin::NoHarvester;

use vars qw( $VERSION $mt_comments_post_handler $mt_hdlr_comment_fields );
$VERSION = '1.2.1';

use MT;
use MT::Plugin;
use MT::Template::Context;


# Create plugin
my $plugin = MT::Plugin->new({
    name => "No Harvester",
    description => 'Makes it impossible to post comments from a cached/harvested page.',
    doc_link => 'http://www.chipple.net/mt/noharvester/',
    author_name => 'Patrick Benny',
    author_link => 'http://www.chipple.net/',
    system_config_template => MT->version_id ge '4' ? 'config.tmpl' : 'config.mt3.tmpl',
    settings => new MT::PluginSettings([
        ['enabled', { Default => '' }],
        ['salt', { Default => GetDefaultSalt() }],
        ['log', { Default => '1' }],
    ]),
    version => $VERSION,
});
MT->add_plugin($plugin);

## For Movable Type 4+
# Use template_source to add "Report" link to edit_comment page
MT->add_callback('MT::App::CMS::template_source.edit_comment', 9, $plugin, \&_template_mt4);
MT->add_callback('MT::App::CMS::template_param.edit_comment', 9, $plugin, \&_param);

## For Movable Type 3.3+
# Use AppTemplateSource to add "Report" link to edit_comment page
MT->add_callback('MT::App::CMS::AppTemplateSource.edit_comment', 9, $plugin, \&_template_mt3);
MT->add_callback('MT::App::CMS::AppTemplateParam.edit_comment', 9, $plugin, \&_param);

## For Movable Type 3.2
# Use BigPAPI to add "Report" link to edit_comment page
MT->add_callback('bigpapi::template::edit_comment', 9, $plugin, \&_template_mt3);
MT->add_callback('bigpapi::param::edit_comment', 9, $plugin, \&_param);

# Remap MT::App::Comments::post to custom function that first checks authenticity
# Keep a pointer to real MT::App::Comments::post to execute it afterwards
$mt_comments_post_handler = \&MT::App::Comments::post;
{
  local($^W) = 0;  # Suppress warning on redefinition
  *MT::App::Comments::post = \&commentPost;
}

# Remap MTCommentFields to custom function that adds our <input/>'s
MT::Template::Context->add_tag(CommentFields => \&hdlrCommentFields);


# Use this in comments, comments_preview and comments_error CGI templates
#   <input type="hidden" name="remote_ip" value="<$MTNoHarvester_RemoteAddr$>" />
#   <input type="hidden" name="auth_check" value="<$MTNoHarvester_AuthCheck$>" />
# and this in PHP templates for individual archives
#   <input type="hidden" name="remote_ip" value="<?=$_SERVER['REMOTE_ADDR']?>" />
#   <input type="hidden" name="auth_check" value="<?=md5($_SERVER['REMOTE_ADDR'].'<$MTEntryID$>9a9a9a9a')?>" />
MT::Template::Context->add_tag(NoHarvester_RemoteAddr => \&GetRemoteAddr);
MT::Template::Context->add_tag(NoHarvester_AuthCheck => \&GetAuthCheckFromCtx);
MT::Template::Context->add_tag(NoHarvester_Salt => sub { GetConfig('salt'); } );


#################
### FILTERING ###
#################

# Check authenticity before continuing to comment post
sub commentPost {
    ##############################
    # From MT::App::Comments::post
    my $app = shift;
    my $q = $app->{query};

    return do_preview($app, $q, @_) if $app->request_method() ne 'POST';

    my $entry_id = $q->param('entry_id')
        or return $app->error("No entry_id");
    # /From MT::App::Comments::post
    ###############################

    if (GetConfig('enabled') ne '1') {
        &$mt_comments_post_handler($app);
        return;
    }

    my $log = GetConfig('log');

    # No remote_ip parameter
    if (!$q->param('remote_ip') || $q->param('remote_ip') eq "") {
        MT::log("Blocked comment: No remote_ip. entry_id = ".$entry_id.", REMOTE_ADDR = ".$app->remote_ip)
            if $log eq '1';
        return $app->handle_error("Cached page?");
    }

    # Different IP
    if ($q->param('remote_ip') ne $app->remote_ip) {
        # Allow same class C IP
        my($cclass_input_ip) = $q->param('remote_ip') =~ /^(.*)\.[0-9]{1,3}$/;
        my($cclass_remote_ip) = $app->remote_ip =~ /^(.*)\.[0-9]{1,3}$/;
        if ($cclass_input_ip ne $cclass_remote_ip) {
            MT::log("Blocked comment: Bad remote_ip '".$q->param('remote_ip')."', ".(VerifyAuthCheck($q,$entry_id)?"Good auth_check":"Bad auth_check").". entry_id = ".$entry_id.", REMOTE_ADDR = ".$app->remote_ip)
                if $log eq '1';
            return $app->handle_error("Cached page? Jumping IPs?");
        }
    }

    # Verify auth_check
    if (!VerifyAuthCheck($q,$entry_id)) {
        MT::log("Blocked comment: Bad auth_check. entry_id = ".$entry_id.", remote_ip = '".$q->param('remote_ip')."', REMOTE_ADDR = ".$app->remote_ip)
            if $log eq '1';
        return $app->error("Spam isn't welcome here. Abuse complaints are regularly filed against offenders.");
    }

    &$mt_comments_post_handler($app);

}

sub hdlrCommentFields {
    my $result = MT::Template::Context::_hdlr_comment_fields(@_);

    return $result if GetConfig('enabled') ne '1';

    my $ctx = shift;

    my $append = '
<input type="hidden" name="remote_ip" value="'.GetRemoteAddr().'" />
<input type="hidden" name="auth_check" value="'.GetAuthCheckFromCtx($ctx).'" />';

    $result =~ s|(<form.+?>)|$1$append|;
    return $result;
}


#################
### UTILITIES ###
#################

# Get auth check value from context
sub GetAuthCheckFromCtx {
    my $ctx = shift;
    my $entry = $ctx->stash('entry');
    return GetAuthCheck($ENV{'REMOTE_ADDR'},$entry->id);
}

# Get auth check value from IP and entry ID
sub GetAuthCheck {
    my($ip,$id) = @_;
    use Digest::MD5;
    my $md5 = Digest::MD5->new;
    $md5->add($ip);
    $md5->add($id);
    $md5->add(GetConfig('salt'));
    return $md5->hexdigest;
}

# Get remote IP address
sub GetRemoteAddr {
    return $ENV{'REMOTE_ADDR'};
}

# Get system config value
sub GetConfig {
    my($item) = @_;
    my $sys = $plugin->get_config_hash('system');
    return $sys->{$item};
}

# Verify whether auth check is valid
sub VerifyAuthCheck {
    my($q,$entry_id) = @_;
    return GetAuthCheck($q->param('remote_ip'),$entry_id) eq $q->param('auth_check');
}

sub GetDefaultSalt {
    # 'Digest::MD5' is to support backward-compatibility with previous version
    # that used Digest::MD5->md5_hex ("function called as class method" warning)
    return Digest::MD5::md5_hex('Digest::MD5'.$ENV{'SCRIPT_FILENAME'});
}


################
### TEMPLATE ###
################

# Get email body text from file
sub GetEmailBody {

    # Load text from reportemail.txt
    use URI::Escape;
    open(REPORTEMAIL, $plugin->envelope."/data/reportemail.txt");
    my $body = "";
    while (my $line = <REPORTEMAIL>) {
        $body .= uri_escape($line);
    }
    close(REPORTEMAIL);

    # Replace <XX> tags by <TMPL_VAR NAME=XX ESCAPE=URL>
    $body =~ s|%3C(.*?)%3E|<TMPL_VAR NAME=$1 ESCAPE=URL>|g;

    return $body;

}

# Add "Whois" and "Report" links next to IP when editing a junked comment
sub _template_mt4 {

    my ($cb, $app, $template) = @_;
    my $body = GetEmailBody();

    # For Movable Type 4
    $$template =~ s|(<mt:var name="ip" escape="html">)|$1<mt:if name="is_junk">&nbsp; <a href="http://ws.arin.net/cgi-bin/whois.pl?queryinput=<mt:var name="ip" escape="url">" target="_blank">Whois</a> &nbsp;<a href="mailto:?subject=Spam%20Report%20IP%20<mt:var name="ip" escape="url">&amp;body=$body">Report</a></mt:if>|;

}

# Add "Whois" and "Report" links next to IP when editing a junked comment (MT3)
sub _template_mt3 {

    my ($cb, $app, $template) = @_;
    my $body = GetEmailBody();

    # For Movable Type 3.2, 3.3+
    $$template =~ s|(<TMPL_VAR NAME=IP ESCAPE=HTML>)|$1<TMPL_IF NAME=IS_JUNK>&nbsp; <a href="http://ws.arin.net/cgi-bin/whois.pl?queryinput=<TMPL_VAR NAME=IP ESCAPE=URL>" target="_blank">Whois</a> &nbsp;<a href="mailto:?subject=Spam%20Report%20IP%20<TMPL_VAR NAME=IP ESCAPE=URL>&amp;body=$body">Report</a></TMPL_IF>|;

}

# Add blog_timezone variable to edit_comment template
sub _param {

    my ($cb, $app, $param) = @_;

    my $blog = MT::Blog->load($param->{blog_id});

    my $so = $blog->server_offset;
    my $partial_hour_offset = 60 * abs($so - int($so));

    $param->{'blog_timezone'} = 
        sprintf("%s%02d%02d", $so < 0 ? '-' : '+',
                abs($so), 
                $partial_hour_offset);
}

1;